#include <Wire.h>
#include <LiquidCrystal.h>

#define SLAVE_ADDRESS 0x63
char retValue[2];
int retValRemain = 0;// remain num of read data
unsigned char cmdbuf[2]; // buffer to command stream
int bufptr=0;  // index to buf prt
int soundState; // 0:off,1:0n
int selLine;//lcd cursor selected line 0~
unsigned int soundFreq;
unsigned int soundDur;
LiquidCrystal lcd(8, 7, 9, 10, 11, 12);

void setup() {
  pinMode(0, OUTPUT); // LED
  pinMode(1, OUTPUT);// LED
  pinMode(5,OUTPUT); // Speaker
  digitalWrite(0, LOW); // set the LED off
  digitalWrite(1, LOW); // set the LED off
  analogReference(DEFAULT);
  soundFreq = 5;
  soundDur = 20;
  soundState = 0;
  // initialize i2c as slave
  Wire.begin(SLAVE_ADDRESS);
  // define callbacks for i2c communication
  Wire.onReceive(receiveData);
  Wire.onRequest(sendData);
}

void loop() {
  if (soundState > 0){
    sound();
    soundState--;
  }
  else{
   // delay(50);
  }
}
void sound(){ // 50ms sound
  float f = 200.0+40.0*soundFreq;
  float t = 0.5/f*1000000.0;
  float d = f*0.005; // 50ms 
  for (int i=0;i<d;i++){
    digitalWrite(5,HIGH);
    delayMicroseconds(t);
    digitalWrite(5,LOW);
    delayMicroseconds(t);
  }
}
// callback for received data
void receiveData(int byteCount){

  while(Wire.available()) {
    char temp = Wire.receive();
    if (bufptr==0 && temp == 0){// 1バイト目の00はスキップする
      continue;
    }

    if (bufptr==0){ // バッファーに１バイト目をストア
      cmdbuf[0] = temp;
      bufptr++;
      continue;
    }
    else{ //2バイト目をストア
      cmdbuf[1] = temp;
      bufptr =0;

      switch(cmdbuf[0]){//コマンドを解析
      case 1:
        switch(cmdbuf[1]){
        case 0:
          digitalWrite(0, LOW); // set the LED1 off
          break;
        case 1:
          digitalWrite(0, HIGH); // set the LED1 on
          break;
        }        
        break;  
      case 2:
        switch(cmdbuf[1]){
        case 0:
          digitalWrite(1, LOW); // set the LED1 off
          break;
        case 1:
          digitalWrite(1, HIGH); // set the LED1 on
          break;
        }
        break;
      case 8:
        switch (cmdbuf[1]){
        case 0:
          soundState=0;
          break;
        case 1:
          lcd.clear();
          lcd.print(String(soundFreq));
          lcd.print(' ');
          lcd.print(String(soundDur));
          soundState = soundDur;
          break;
        }
        break;
      case 9:
        soundFreq=(unsigned int)cmdbuf[1];
        break;
      case 10:
        soundDur=(unsigned int)cmdbuf[1];
        break;
      case 20:
        switch(cmdbuf[1]){
        case 2:
          pinMode(2,INPUT);
          break;
        case 3:
          pinMode(3,INPUT);
          break;
        case 4:
          pinMode(4,INPUT);
          break;
        case 6:
          pinMode(6,INPUT);
          break;
        case 13:
          pinMode(13,INPUT);
          break;
        }
        break;
      case 21:
        switch(cmdbuf[1]){
        case 2:
          pinMode(2,OUTPUT);
          break;
        case 3:
          pinMode(3,OUTPUT);
          break;
        case 4:
          pinMode(4,OUTPUT);
          break;
        case 6:
          pinMode(6,OUTPUT);
          break;
        case 13:
          pinMode(13,OUTPUT);
          break;
        }
        break;  
      case 22:
        switch(cmdbuf[1]){
        case 2:
          digitalWrite(2,HIGH);
          break;
        case 3:
          digitalWrite(3,HIGH);
          break;
        case 4:
          digitalWrite(4,HIGH);
          break;
        case 6:
          digitalWrite(6,HIGH);
          break;
        case 13:
          digitalWrite(13,HIGH);
          break;
        }
        break;
      case 23:
        switch(cmdbuf[1]){
        case 2:
          digitalWrite(2,LOW);
          break;
        case 3:
          digitalWrite(3,LOW);
          break;
        case 4:
          digitalWrite(4,LOW);
          break;
        case 6:
          digitalWrite(6,LOW);
          break;
        case 13:
          digitalWrite(13,LOW);
          break;
        }
        break;
      case 24:
        switch(cmdbuf[1]){
        case 2:
          retValue[0]=digitalRead(2);
          retValue[1]=0;
          retValRemain=1;
          break;
        case 3:
          retValue[0]=digitalRead(3);
          retValue[1]=0;
          retValRemain=1;
          break;
        case 4:
          retValue[0]=digitalRead(4);
          retValue[1]=0;
          retValRemain=1;
          break;
        case 6:
          retValue[0]=digitalRead(6);
          retValue[1]=0;
          retValRemain=1;
          break;
        case 13:
          retValue[0]=digitalRead(13);
          retValue[1]=0;
          retValRemain=1;
          break;        
        }
        break;
      case 25:
        analogWrite(3,cmdbuf[1]);
        break;
      case 26:
        analogWrite(6,cmdbuf[1]);
        break;
       
      case 30:
        lcd.clear();
        int value;
        switch(cmdbuf[1]){
        case 0:
          value = analogRead(0);
          retValue[0]=B11111111 & value;
          retValue[1]=value >> 8;
          retValRemain = 2;
          lcd.print("ADC");
          lcd.print(String(value));
          break;
        case 1:
          value = analogRead(1);
          retValue[0]=B11111111 & value;
          retValue[1]=value >> 8;
          retValRemain = 2;
          break;
        }
        break;
        
      case 50:
        switch(cmdbuf[1]){
        case 16:
          lcd.begin(16, 2);
          break;
        case 20:
          lcd.begin(20,4);
          break;
        }
        break;
      case 51:
        lcd.clear();
        break;
      case 52:
        switch(cmdbuf[1]){
        case 0:
          lcd.noCursor();
          lcd.noBlink();
          break;
        case 1:
          lcd.cursor();
          lcd.noBlink();
          break;
        case 2:
          lcd.noCursor();
          lcd.blink();
          break;
        case 3:
          lcd.cursor();
          lcd.blink();
          break;
        }
        break;
      case 53:
        lcd.print(cmdbuf[1]);
        break;
      case 54:
        lcd.setCursor(cmdbuf[1],selLine);
        break;
      case 55:
        selLine = cmdbuf[1];
        break;
      case 56:
        switch(cmdbuf[1]){
        case 0:
          lcd.noDisplay();
          break;
        case 1:
          lcd.display();
          break;
        }
        break;
      case 57:
        switch(cmdbuf[1]){
        case 0:
          lcd.scrollDisplayLeft();
          break;
        case 1:
          lcd.scrollDisplayRight();
          break;
        }
        break;
      case 58:
        switch(cmdbuf[1]){
        case 0:
          lcd.noAutoscroll();
          break;
        case 1:
          lcd.autoscroll();
          break;
        }
        break;
      }
    }
  }
}
// callback for sending data
void sendData(){
  if(retValRemain>0){
    retValRemain --;
    Wire.send(retValue[retValRemain]);
  }    else {
    Wire.send(0);
  }
}



